use std::net::SocketAddr;

use notmad::Mad;

use crate::{grpc_server::GrpcServer, state::ServerState};

#[derive(clap::Parser)]
pub struct ServeCommand {
    #[arg(
        long = "grpc-host",
        env = "NORUN_GRPC_HOST",
        default_value = "127.0.0.1:4242"
    )]
    grpc_host: SocketAddr,
}

impl ServeCommand {
    pub async fn execute(&self) -> anyhow::Result<()> {
        let state = ServerState {};

        Mad::builder()
            .add(GrpcServer {
                host: self.grpc_host,
                state: state.clone(),
            })
            .run()
            .await?;

        Ok(())
    }
}
