use std::net::SocketAddr;

use norun_grpc_interface::registry_service_server::RegistryServiceServer;
use notmad::MadError;
use tokio_util::sync::CancellationToken;

use crate::{grpc_server::registry::GrpcRegistryService, state::ServerState};

mod registry;

pub struct GrpcServer {
    pub host: SocketAddr,
    pub state: ServerState,
}

impl GrpcServer {
    pub async fn serve(&self, cancellation_token: CancellationToken) -> anyhow::Result<()> {
        tracing::info!("serving grpc on {}", self.host);

        tonic::transport::Server::builder()
            .add_service(RegistryServiceServer::new(GrpcRegistryService {
                state: self.state.clone(),
            }))
            .serve_with_shutdown(
                self.host,
                async move { cancellation_token.cancelled().await },
            )
            .await?;

        Ok(())
    }
}

#[async_trait::async_trait]
impl notmad::Component for GrpcServer {
    fn name(&self) -> Option<String> {
        Some("norun/gprc-server".into())
    }

    async fn run(&self, cancellation_token: CancellationToken) -> Result<(), MadError> {
        self.serve(cancellation_token)
            .await
            .map_err(MadError::Inner)?;

        Ok(())
    }
}
