use std::{
    collections::BTreeMap,
    sync::{Arc, LazyLock},
};

use norun_grpc_interface::Project;
use tokio::sync::Mutex;

use crate::state::ServerState;

#[derive(Clone)]
pub struct RegistryService {
    store: Arc<Mutex<Vec<Project>>>,
}

impl RegistryService {
    pub async fn store(&self, project: &Project) -> anyhow::Result<()> {
        tracing::debug!("storing project");

        let mut store = self.store.lock().await;

        store.push(project.clone());

        tracing::debug!("currently has {} in storage", store.len());

        Ok(())
    }
}

pub trait RegistryServiceState {
    fn registry_service(&self) -> RegistryService;
}

impl RegistryServiceState for ServerState {
    fn registry_service(&self) -> RegistryService {
        static SERVICE: LazyLock<RegistryService> = LazyLock::new(|| RegistryService {
            store: Arc::default(),
        });

        SERVICE.clone()
    }
}
